/*
m_3d_atmo - Fill a room with a background atmosphere

DESCRIPTION
Fill a room with a background atmosphere, by reading a stereo sample of a
recording, twice with different offsets and placing the channels in the four
corners of the room.

SYNTAX
aleft, aright, aoutsrc m_3d_atmo klistenerx, klistenery, klistenerz, kheadrot, \
	isoundfn, istart, iroomfn

INITIALIZATION
isoundfn - Number of the ftable holding the stereo atmosphere recording
istart - Start in seconds for the second copy to be looped alongside the
	original
iroomfn - Number of the ftable holding hrtfearly/reverb room parameters

PERFORMANCE
klistenerx, klistenery, klistenerz - x, y and z coordinate of the listener in
	the room
kheadrot - Head rotation of the listener

CREDITS
Author: Jeanette C., with much support from the Csound community. Special thanks
to Brian Carty, the author of hrtfearly and hrtferverb.
*/

opcode m_3d_atmo, aaa, kkkkiii
	klx, kly, klz, kheadrot, isound, istart, iroomfn xin

	; Calculate loop start and end for both looping copies
	iloopend = nsamp(isound) / sr
	if iloopend < 15 then
		icross = iloopend / 3
	else
		icross = 5
	endif

	if iloopend < 1.2 then
		ifade = .2
	else
		ifade = 1
	endif

	kenv expseg .0001, .1, 1, p3-(.1 + ifade), 1, ifade, .0001

	ai11, ai12 flooper2 1, 1, 0, iloopend, icross, isound, 0, 0, 0, 0
	ai21, ai22 flooper2 1, 1, 0, iloopend, icross, isound, istart, 0, 0, 0

	; Read room parameters
	irfade table 0, iroomfn, 0
	irorder table 1, iroomfn, 0
	irx table 2, iroomfn, 0
	iry table 3, iroomfn, 0
	irz table 4, iroomfn, 0
	irwallhi table 5, iroomfn, 0
	irwalllo table 6, iroomfn, 0
	irwallg1 table 7, iroomfn, 0
	irwallg2 table 8, iroomfn, 0
	irwallg3 table 9, iroomfn, 0
	irfloorhi table 10, iroomfn, 0
	irfloorlo table 11, iroomfn, 0
	irfloorg1 table 12, iroomfn, 0
	irfloorg2 table 13, iroomfn, 0
	irfloorg3 table 14, iroomfn, 0
	irceilhi table 15, iroomfn, 0
	irceillo table 16, iroomfn, 0
	irceilg1 table 17, iroomfn, 0
	irceilg2 table 18, iroomfn, 0
	irceilg3 table 19, iroomfn, 0

	if irz < 1.8 then
		iaz = irz / 2
	else
		iaz = 1.6
	endif

	; position the sound
	Sleft sprintf "hrtf-%d-left.dat", sr
	Sright sprintf "hrtf-%d-right.dat", sr
	al1, ar1, irtl1, irth1, imfp1 hrtfearly ai11, .1, .1, iaz, klx, kly, klz, Sleft, Sright, 0, irfade, sr, irorder, 1, kheadrot, irx, iry, irz, irwallhi, irwalllo, irwallg1, irwallg2, irwallg3, irfloorhi, irfloorlo, irfloorg1, irfloorg2, irfloorg3, irceilhi, irceillo, irceilg1, irceilg2, irceilg3
	al2, ar2, irtl2, irth2, imfp2 hrtfearly ai21, (irx - .1), .1, iaz, klx, kly, klz, Sleft, Sright, 0, irfade, sr, irorder, 1, kheadrot, irx, iry, irz, irwallhi, irwalllo, irwallg1, irwallg2, irwallg3, irfloorhi, irfloorlo, irfloorg1, irfloorg2, irfloorg3, irceilhi, irceillo, irceilg1, irceilg2, irceilg3
	al3, ar3, irtl3, irth3, imfp3 hrtfearly ai12, (irx - .1), (iry - .1), iaz, klx, kly, klz, Sleft, Sright, 0, irfade, sr, irorder, 1, kheadrot, irx, iry, irz, irwallhi, irwalllo, irwallg1, irwallg2, irwallg3, irfloorhi, irfloorlo, irfloorg1, irfloorg2, irfloorg3, irceilhi, irceillo, irceilg1, irceilg2, irceilg3
	al4, ar4, irtl4, irth4, imfp4 hrtfearly ai22, .1, (iry - .1), iaz, klx, kly, klz, Sleft, Sright, 0, irfade, sr, irorder, 1, kheadrot, irx, iry, irz, irwallhi, irwalllo, irwallg1, irwallg2, irwallg3, irfloorhi, irfloorlo, irfloorg1, irfloorg2, irfloorg3, irceilhi, irceillo, irceilg1, irceilg2, irceilg3

	; combine signals and output
	al = (al1 + al2 + al3 + al4) * .25
	ar = (ar1 + ar2 + ar3 + ar4) * .25
	; prints "\nirtl: %f, irth: %f, imfp: %f", irtl1, irth1, imfp1
	aoutsrc = (al1 + al2 + ar1 + ar2)
	xout al, ar, aoutsrc
endop
