/*
m_3d_move - create a movement in 3d space

DESCRIPTION
Create a movement in 3d space, based on a movement ftable with coordinates,
movement time/speed and movement curve.
NOTE: You MUST also include m_segk.udo and supply three function tables with
curves for it. Usually something like this would do:
f1 0 4096 23 "log.ftalbe"
f2 0 4096 7 0 4096 1
f3 0 4096 23 "exp.ftable"
and then give 1, 2 and 3 for ifn1, ifn2 and ifn3 respectively.

SYNTAX
kx, ky, kz m_3d_move imovefn, ifn1, ifn2, ifn3 [, iroomfn]

INITIALIZATION
imovefn - ftable containing the movement profile. The format is:
	x = x coordinate in meters
	y = y coordinate in meters
	z = z coordinate in meters
	movement time/speed = if positive this is n absolute time, if negative
		it is the average speed from points to point. Speed is given in km/h
		(kilometres per hour)
	curve_type = type of the curve 0-2, relating to ifn1, ifn2 and ifn3,
		usually logarithmic, linear and exponential.
	The time/speed and curve_type of the first point will be ignored!
ifn1, ifn2, ifn3 - ftables for the three curve types
iroomfn (optional) - if specified, this is an ftable describing a room (see
	m_3d_atmo UDO for more details)

PERFORMANCE
kx, ky, kz - Current x, y and z coordinate of the movement

CREDITS
Author: Jeanette C.
*/

opcode m_3d_move, kkk, iiiij
	imovefn, ifn1, ifn2, ifn3, iroomfn xin

	; Set up basic parameters
	inumpoints = ftlen(imovefn) / 5 ; 5 values per point in the table
	kx init 0
	ky init 0
	kz init 0
	kxstart init .1
	kystart init .1
	kzstart init .1
	kxend init tab_i(0, imovefn, 0)
	kyend init tab_i(1, imovefn, 0)
	kzend init tab_i(2, imovefn, 0)
	kmove init tab_i(8, imovefn, 0)
	kcurve init tab_i(9, imovefn, 0)
	kfin init 0 ; end-marker of the segment generator
	ktrig init 1 ; trigger for the segment generator
	kndx init 1 ; index for the current point

	if iroomfn != -1 then ; if a room is supplied, store dimensions
		iroomx tab_i 2, iroomfn, 0
		iroomy tab_i 3, iroomfn, 0
		iroomz tab_i 4, iroomfn, 0
	else
		iroomx init 0
		iroomy init 0
		iroomz init 0
	endif

	if kfin == 1 then
		if kndx < inumpoints then ; if there is a new point
			kxstart = kxend
			kystart = kyend
			kzstart = kzend
			kxend table (kndx * 5), imovefn, 0
			kyend table (kndx * 5) +1, imovefn, 0
			kzend table (kndx * 5) +2, imovefn, 0
			kmove table (kndx * 5) +3, imovefn, 0
			kcurve table (kndx * 5) +4, imovefn, 0

			if kmove < 0 then ; kmove is a speed in km/h, calculate time
				kdistance = sqrt((kxend - kxstart)^2 + (kyend - kystart)^2 + (kzend - kzstart)^2)
				ktime = kdistance * 3.6 / (-1 * kmove)
				printks "\nPoint %d to %d, time: %f", 1/kr, kndx, (kndx+1), ktime
			else
				ktime = kmove
			endif
			kndx += 1
			ktrig = 1
			kfin = 0
		endif
	endif

	kx, kfin m_segk kxstart, ktime, kxend, kcurve, ktrig, ifn1, ifn2, ifn3
	ky, kfin m_segk kystart, ktime, kyend, kcurve, ktrig, ifn1, ifn2, ifn3
	kz, kfin m_segk kzstart, ktime, kzend, kcurve, ktrig, ifn1, ifn2, ifn3

	if iroomfn != -1 then ; Check that we don't bump into the walls,
		; a security distance of .1 is advised!
		if kx < .1 then
			printks "\nx coordinate too far left! x = %f", 1/kr, kx
			kx = .1
		else
			if (iroomx - kx) < .09 then
				printks "\nx coordinate too far right! x = %f", 1/kr, kx
				kx = (iroomx - .1)
			endif
		endif

		if ky < .1 then
			printks "\ny cooreindate too far ahead! y = %f", 1/kr, ky
			ky = .1
		else
			if (iroomy - ky) < .09 then
				printks "\ny coordinate too far back! y = %f", 1/kr, ky
				ky = (iroomy - .1)
			endif
		endif

		if kz < .1 then
			printks "\nz coordinate too low! z = %f", 1/kr, kz
			kz = .1
		else
			if (iroomz - kz) < .09 then
				printks "\nz coordinate too high! z = %f", 1/kr, kz
				kz = (iroomz - .1)
			endif
		endif
	endif

	xout kx, ky, kz
	ktrig = 0
endop
